<?php
/**
 * Copyright (C) 2013 OpenMediaVault Plugin Developers
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

require_once ("openmediavault/object.inc");
require_once ("openmediavault/config.inc");
require_once ("openmediavault/error.inc");
require_once ("openmediavault/util.inc");
require_once ("openmediavault/rpcservice.inc");
require_once ("openmediavault/notify.inc");

class OMVRpcServiceVdr extends OMVRpcServiceAbstract
{
    const COUNTRY_LIST   = "c",
          SATELLITE_LIST = "s";

    private $_channelFileLocation = "/var/lib/vdr/channels.conf";

    private function getXPath()
    {
        return '//services/vdr';
    }

    public function getName()
    {
        return "VDR";
    }

    public function initialize()
    {
        $this->registerMethod("getSettings");
        $this->registerMethod("setSettings");
        $this->registerMethod("scanChannels");
        $this->registerMethod("scanInProgress");
        $this->registerMethod("getChannels");
        $this->registerMethod("setChannels");
        $this->registerMethod("checkDevices");
        $this->registerMethod("enumerateCountries");
        $this->registerMethod("enumerateSatellites");
    }

    public function getSettings($params, $context)
    {
        global $xmlConfig;

        // Validate the RPC caller context.
        $this->validateMethodContext($context, array("role" => OMV_ROLE_ADMINISTRATOR));

        // Get the configuration object.
        $object = $xmlConfig->get($this->getXPath());

        if (is_null($object))
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $this->getXPath());

        // Modify the result object. (Change characters to values)
        $object['vdr_enable'] = boolval($object['vdr_enable']);
        $object['vdr_logging_on'] = boolval($object['vdr_logging_on']);
        //['vdr_transpondertype'];
        //vdr_atsc_country
        //vdr_atsc_type
        $object['vdr_atsc_scan_encrypted'] = boolval($object['vdr_atsc_scan_encrypted']);
        //vdr_dvb_t_country
        $object['vdr_dvb_t_scan_encrypted'] = boolval($object['vdr_dvb_t_scan_encrypted']);
        //vdr_dvb_c_country
        //$object['vdr_dvb_c_qam'] = intval ($object['vdr_dvb_c_qam']);
        //$object['vdr_dvb_c_symbolrate'] = intval ($object['vdr_dvb_c_symbolrate']);
        $object['vdr_dvb_c_extended_qam'] = boolval($object['vdr_dvb_c_extended_qam']);
        $object['vdr_dvb_c_scan_encrypted'] = boolval($object['vdr_dvb_c_scan_encrypted']);
        //vdr_dvb_s_country
        //vdr_dvb_s_satellite
        $object['vdr_dvb_s_scan_encrypted'] = boolval($object['vdr_dvb_s_scan_encrypted']);

        //vdr_recordingdir
        $object['vdr_maxfilesize_gb'] = intval($object['vdr_maxfilesize_gb']);
        $object['vdr_subtitles'] = boolval($object['vdr_subtitles']);
        //vdr_subtitle_languages
        //vdr_epglanguage
        //vdr_channelupdatemode

        $object['vdr_streamdev_enable'] = boolval($object['vdr_streamdev_enable']);
        $object['vdr_streamdev_port'] = intval($object['vdr_streamdev_port']);
        //vdr_streamdev_hosts

        $object['vdr_live_enable'] = boolval($object['vdr_live_enable']);
        $object['vdr_live_port'] = intval($object['vdr_live_port']);

        $object['vdradminam_enable'] = boolval($object['vdradminam_enable']);
        $object['vdradminam_port'] = intval($object['vdradminam_port']);

        return $object;
    }

    public function setSettings($params, $context)
    {
        global $xmlConfig;

        // Validate the RPC caller context.
        $this->validateMethodContext($context, array("role" => OMV_ROLE_ADMINISTRATOR));

        $this->validateMethodParams($params, '{
            "type":"object",
            "properties":{
                "vdr_enable":{"type":"boolean"},
                "vdr_logging_on":{"type":"boolean"},
                "vdr_transpondertype":{"type":"string"},
                "vdr_atsc_country":{"type":"string"},
                "vdr_atsc_type":{"type":"number"},
                "vdr_atsc_scan_encrypted":{"type":"boolean"},
                "vdr_dvb_t_country":{"type":"string"},
                "vdr_dvb_t_scan_encrypted":{"type":"boolean"},
                "vdr_dvb_c_country":{"type":"string"},
                "vdr_dvb_c_qam":{"type":"string", "maxLength":"1"},
                "vdr_dvb_c_symbolrate":{"type":"string", "maxLength":"1"},
                "vdr_dvb_c_extended_qam":{"type":"boolean"},
                "vdr_dvb_c_scan_encrypted":{"type":"boolean"},
                "vdr_dvb_s_country":{"type":"string"},
                "vdr_dvb_s_satellite":{"type":"string"},
                "vdr_dvb_s_scan_encrypted":{"type":"boolean"},
                "vdr_recordingdir":{"type":"string"},
                "vdr_maxfilesize_gb":{"type":"integer","minimum":1},
                "vdr_subtitles":{"type":"boolean"},
                "vdr_subtitle_languages":{"type":"string"},
                "vdr_epglanguage":{"type":"string"},
                "vdr_channelupdatemode":{"type":"number","minimum":0,"maximum":6},
                "vdr_streamdev_enable":{"type":"boolean"},
                "vdr_streamdev_port":{"type":"integer","minimum":1,"maximum":65535},
                "vdr_streamdev_hosts":{"type":"string"},
                "vdr_live_enable":{"type":"boolean"},
                "vdr_live_port":{"type":"integer","minimum":1,"maximum":65535},
                "vdradminam_enable":{"type":"boolean"},
                "vdradminam_port":{"type":"integer","minimum":1,"maximum":65535}
            }
        }');

        // Get the existing configuration object.
        $oldObject = $xmlConfig->get($this->getXPath());

        if (is_null($oldObject))
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $this->getXPath());

        // Update the configuration object.
        $object = array(
            "vdr_enable" => array_boolval($params, 'vdr_enable'),
            "vdr_logging_on" => array_boolval($params, 'vdr_logging_on'),
            "vdr_transpondertype" => $params['vdr_transpondertype'],
            "vdr_atsc_country" => $params['vdr_atsc_country'],
            "vdr_atsc_type" => $params['vdr_atsc_type'],
            "vdr_atsc_scan_encrypted" => array_boolval($params, 'vdr_atsc_scan_encrypted'),
            "vdr_dvb_t_country" => $params['vdr_dvb_t_country'],
            "vdr_dvb_t_scan_encrypted" => array_boolval($params, 'vdr_dvb_t_scan_encrypted'),
            "vdr_dvb_c_country" => $params['vdr_dvb_c_country'],
            "vdr_dvb_c_qam" => $params['vdr_dvb_c_qam'],
            "vdr_dvb_c_symbolrate" => $params['vdr_dvb_c_symbolrate'],
            "vdr_dvb_c_extended_qam" => array_boolval($params, 'vdr_dvb_c_extended_qam'),
            "vdr_dvb_c_scan_encrypted" => array_boolval($params, 'vdr_dvb_c_scan_encrypted'),
            "vdr_dvb_s_country" => $params['vdr_dvb_s_country'],
            "vdr_dvb_s_satellite" => $params['vdr_dvb_s_satellite'],
            "vdr_dvb_s_scan_encrypted" => array_boolval($params, 'vdr_dvb_s_scan_encrypted'),
            "vdr_recordingdir" => $params['vdr_recordingdir'],
            "vdr_maxfilesize_gb" => $params['vdr_maxfilesize_gb'],
            "vdr_subtitles" => $params['vdr_subtitles'],
            "vdr_subtitle_languages" => $params['vdr_subtitle_languages'],
            "vdr_epglanguage" => $params['vdr_epglanguage'],
            "vdr_channelupdatemode" => $params['vdr_channelupdatemode'],
            "vdr_streamdev_enable" => array_boolval($params, 'vdr_streamdev_enable'),
            "vdr_streamdev_port" => $params['vdr_streamdev_port'],
            "vdr_streamdev_hosts" => $params['vdr_streamdev_hosts'],
            "vdr_live_enable" => array_boolval($params, 'vdr_live_enable'),
            "vdr_live_port" => $params['vdr_live_port'],
            "vdradminam_enable" => array_boolval($params, 'vdradminam_enable'),
            "vdradminam_port" => $params['vdradminam_port']
        );

        // Update the xmlConfig file. If it fails then generate an error
        if ($xmlConfig->replace($this->getXPath(), $object) === false)
            throw new OMVException(OMVErrorMsg::E_CONFIG_SET_OBJECT_FAILED);

        // Notify configuration changes.
        $dispatcher = &OMVNotifyDispatcher::getInstance();
        $dispatcher->notify(OMV_NOTIFY_MODIFY, "org.openmediavault.services.vdr", $object);

        // Return the configuration object.
        return $object;
    }

    /**
     * Performs a channel scan
     *
     * @return void
     */
    public function scanChannels($params, $context)
    {
        $cmd = "";

        // Check if scan is active
        // If active or checking failed return
        if ($this->scanInProgress(array(), $context))
            return;

        // Temporary file
        $channelFile = "/tmp/channels.conf";

        // Output file type is for VDR 1.7
        $outputMode = " -o 7 ";

        // Parse script based on transponder settings
        $transpoderType = $params['vdr_transpondertype'];

        switch ($transpoderType) {
            case "a":
                if (boolval($params['vdr_atsc_scan_encrypted']) == false) {
                    $vdr_atsc_scan_encrypted = " -E 0";
                } else {
                    $vdr_atsc_scan_encrypted = "";
                }

                $str = " -c "
                     . $params['vdr_atsc_country']
                     . " -A "
                     . $params['vdr_atsc_type']
                     . $vdr_atsc_scan_encrypted;

                $cmd = "w_scan -fa"
                     . $str
                     . $outputMode
                     . " >> "
                     . $channelFile;

                break;

            case "c":
                if ($params['vdr_dvb_c_qam'] == " ") {
                    $vdr_dvb_c_qam = "";
                } else {
                    $vdr_dvb_c_qam = " -Q " . $params['vdr_dvb_c_qam'];
                }

                if ($params['vdr_dvb_c_symbolrate'] == " ") {
                    $vdr_dvb_c_symbolrate = "";
                } else {
                    $vdr_dvb_c_symbolrate = " -S " . $params['vdr_dvb_c_symbolrate'];
                }

                if (boolval($params['vdr_dvb_c_extended_qam']) == false) {
                    $vdr_dvb_c_extended_qam = "";
                } else {
                    $vdr_dvb_c_extended_qam = " -e 2";
                }

                if (boolval($params['vdr_dvb_c_scan_encrypted']) == false) {
                    $vdr_dvb_c_scan_encrypted = " -E 0";
                } else {
                    $vdr_dvb_c_scan_encrypted = "";
                }

                $str = " -c " . $params['vdr_dvb_c_country']
                     . $vdr_dvb_c_qam . $vdr_dvb_c_symbolrate
                     . $vdr_dvb_c_extended_qam
                     . $vdr_dvb_c_scan_encrypted;

                $cmd = "w_scan -fc"
                     . $str
                     . $outputMode
                     . " >> "
                     . $channelFile;

                break;

            case "t":
                if (boolval($params['vdr_dvb_t_scan_encrypted']) == false) {
                    $vdr_dvb_t_scan_encrypted = " -E 0";
                } else {
                    $vdr_dvb_t_scan_encrypted = "";
                }

                $str = " -c "
                     . $params['vdr_dvb_t_country']
                     . $vdr_dvb_t_scan_encrypted;
                $cmd = "w_scan -ft"
                     . $str
                     . $outputMode
                     . " >> "
                     . $channelFile;

                break;

            case "s":
                if (boolval($params['vdr_dvb_s_scan_encrypted']) == false) {
                    $vdr_dvb_s_scan_encrypted = " -E 0";
                } else {
                    $vdr_dvb_s_scan_encrypted = "";
                }

                $str = " -c "
                     . $params['vdr_dvb_s_country']
                     . " -s "
                     . $params['vdr_dvb_s_satellite']
                     . $vdr_dvb_s_scan_encrypted;

                $cmd = "w_scan -fs"
                     . $str
                     . $outputMode
                     . " >> "
                     . $channelFile;

                break;
        }

        $cmd = sprintf("%1\$s; cat %2\$s > %3\$s; rm %2\$s;",
            $cmd, $channelFile, $this->_channelFileLocation);

        // If script fails to execute then create an error
        if (0 !== $this->exec($cmd, $output))
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED,
                $cmd, implode("\n", $output));
    }

    /**
     * Check if w_scan in running
     *
     * @return boolean
     */
    public function scanInProgress($params, $context)
    {
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array("role" => OMV_ROLE_ADMINISTRATOR));

        $cmd = "pgrep -n w_scan 2>&1";
        $pgrepReturn = $this->exec($cmd, $output);

        if ($pgrepReturn === 0) {
            // "Channel scan is still running"
            return true;
        } else if ($pgrepReturn === 1) {
            // "Channel Scan is not active"
            return false;
        } else {
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED,
                $cmd, implode("\n", $output));
        }
    }

    /**
     * Get channels from channels.conf
     *
     * @return array
     */
    public function getChannels($params, $context)
    {
        $cmd = sprintf("cat %s", $this->_channelFileLocation);
        if (0 !== $this->exec($cmd, $output))
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED,
                $cmd, implode("\n", $output));

        $retArray = array();

        foreach ($output as $chanNbr => $chanVal) {

            $channelPortions = explode(":", $chanVal);
            $channelNameCompany = explode(";", $channelPortions[0]);

            $channelEncrypted = $channelPortions[8] != "0";

            $retArray[$chanNbr] = array(
                "channel"          => $chanVal,
                "channelName"      => $channelNameCompany[0],
                "channelCompany"   => $channelNameCompany[1],
                "channelEncrypted" => $channelEncrypted
            );
        }

        return $retArray;
    }

    /**
     * Save channels to channels.conf
     */
    public function setChannels($params, $context)
    {
        $channelString = "";

        foreach ($params['channels'] as $paramsk => $paramsv) {
            $channelString = $channelString . $paramsv . "\\n";
        }

        $channelString = substr($channelString, 0, -2);

        $cmd = "/etc/init.d/vdradmin-am stop; "
             . "/etc/init.d/vdr stop; "
             . "echo '"
             . $channelString
             . "' > "
             . $this->_channelFileLocation . ";"
             . "/etc/init.d/vdr start;"
             . "/etc/init.d/vdradmin-am start;";

        if (0 !== $this->exec($cmd, $output))
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED,
                $cmd, implode("\n", $output));
    }

    public function checkDevices($params, $context)
    {
        // Validate the RPC caller context.
        $this->validateMethodContext($context,
            array("role" => OMV_ROLE_ADMINISTRATOR));

        $cmd = "if [ -d /dev/dvb ]; then ls /dev/dvb; fi";

        // If script fails to execute then create an error
        if (0 !== $this->exec($cmd, $output))
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED,
                $cmd, implode("\n", $output));

        return count($output);
    }

    public function enumerateCountries($params, $context)
    {
        // Validate the RPC caller context.
        $this->validateMethodContext($context,
            array("role" => OMV_ROLE_ADMINISTRATOR));

        return $this->getWscanList(self::COUNTRY_LIST);
    }

    public function enumerateSatellites($params, $context)
    {
        // Validate the RPC caller context.
        $this->validateMethodContext($context,
            array("role" => OMV_ROLE_ADMINISTRATOR));

        return $this->getWscanList(self::SATELLITE_LIST);
    }

    private function getWscanList($type)
    {
        $cmd = sprintf("w_scan -%s? 2>&1", $type);

        if (0 !== $this->exec($cmd, $output))
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED,
                $cmd, implode("\n", $output));

        $list = array();

        foreach ($output as $row) {
            $row = trim($row);
            $row = str_replace("\t\t", "\t", $row);
            $data = explode("\t", $row);

            if (self::COUNTRY_LIST === $type) {
                $list[] = array(
                    "abbr" => $data[0],
                    "country" => $data[1],
                );
            } else if (self::SATELLITE_LIST === $type) {
                $list[] = array(
                    "abbr" => $data[0],
                    "satellite" => $data[1],
                );
            }
        }

        return $list;
    }

}

// Register the RPC service.
$rpcServiceMgr = &OMVRpcServiceMgr::getInstance();
$rpcServiceMgr->registerService(new OMVRpcServiceVdr());
